'use strict';

const DeployService = require('./lib/deploy-service');

class ServerlessLambdaAliasPlugin {
  constructor(serverless, options) {
    this.serverless = serverless;
    this.options = options;
    this.deploy = new DeployService();
    this.provider = this.serverless.getProvider('aws');

    this.commands = {
      'update-alias': {
        usage: 'Get the version from one alias and update a target alias',
        lifecycleEvents: ['exec'],
        options: {
          from: {
            usage: 'Specify the source alias to read the version or you can use $LATEST',
            shortcut: 'f',
            required: true,
          },
          to: {
            usage: "Specify the target alias to be updated, if it doesn't exist, will create it",
            shortcut: 't',
            required: true,
          },
        },
      },
      'list-aliases': {
        usage: 'List each alias with their respective version number',
        lifecycleEvents: ['exec'],
      },
    };

    this.hooks = {
      'update-alias:exec': this.updateAliasHandler.bind(this),
      'list-aliases:exec': this.listAliasesHandler.bind(this),
    };
  }

  get getAllFunctionsNames() {
    return this.serverless.service.getAllFunctionsNames();
  }

  async updateAliasHandler() {
    const functions = this.getAllFunctionsNames;
    const { from, to } = this.options;

    for (const fnName of functions) {
      const result = await this.deploy.updateAlias(fnName, from, to);

      this.serverless.cli.log(`Updated alias ${result.to} of ${result.name} to version ${result.fromVersion}`);
    }
  }

  async listAliasesHandler() {
    const functions = this.getAllFunctionsNames;

    for (const fnName of functions) {
      const result = await this.deploy.getAliases(fnName);

      this.serverless.cli.log(`Alias(es) for function ${fnName}. ${Object.keys(result).length} alias(es) found`);
      for (const alias in result) {
        if (result.hasOwnProperty(alias)) {
          this.serverless.cli.log(`Alias name: ${alias}. Version: ${result[alias]}`);
        }
      }
    }
  }
}

module.exports = ServerlessLambdaAliasPlugin;
